<?php
namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\GER\Services\CrudService;
use Doctrine\DBAL\Connection;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DataSource\FilterCriteria;


class Tco01900UtilizacaoProdutoReceita {
    protected $tco01900UtilizacaoProdutoReceitaService;
    protected $connection;
    protected $environment;
    protected $crudservice;
    protected $parameterbag;
    protected $dataSourceManager;
    protected $tcoUtil;
    protected $util;
  
    public function __construct($crudservice, $parameterbag, $entityManager, $environment, $dataSourceManager, $tcoUtil, $util, $tco01900UtilizacaoProdutoReceitaService){
        $this->crudservice   = $crudservice;
        $this->parameterbag  = $parameterbag;
        $this->entityManager = $entityManager;
        $this->connection    = $entityManager->getConnection();
        $this->environment   = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->tcoUtil = $tcoUtil; 
        $this->util = $util;
        $this->tco01900UtilizacaoProdutoReceitaService = $tco01900UtilizacaoProdutoReceitaService;

        //parent::__construct($crudservice, $parameterbag);
  }

    
    public function geraListaServico($request, $response){
        try{
            $arrayCodLista = array();
            $this->entityManager->beginTransaction();
            $row = $request->getRow();
            $filial = $row['CDFILIAL'];
            $servico = $row['CDSERVFILI'];
            $selecionados = is_array($servico);
            if($selecionados){
                $selecionados = ( count($servico) > 0 );
            }
            if ($selecionados){
                $codLista = $this->util->insertLista($servico);
                array_push($arrayCodLista,$codLista);
            }else{
                $todosServicos = $this->tco01900UtilizacaoProdutoReceitaService->todosServicos($filial);
                $codLista = $this->util->insertLista($todosServicos);
                array_push($arrayCodLista,$codLista);
            }
            
            $this->entityManager->commit();
            $response->addDataset(new DataSet('LISTA', $arrayCodLista));
        }catch (\Exception $error) {
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($error->getMessage(), 1));
        }
    }
    
    public function retornaProdutoPorPratos($request,$response){
        $row = $request->getRow();
        $produtos = $this->tco01900UtilizacaoProdutoReceitaService->retornaProdutoPorPratos($row);
        $response->addDataset(new DataSet('PRODUTOS', array($produtos)));
    }
    
    public function deleteCheckedRows($request, $response){
        try{
            $this->entityManager->beginTransaction();
            $rows = $request->getRow();
            foreach($rows as $receita){
                $this->deletaReceita($receita);    
            }
            $this->entityManager->commit();
        }catch(\Exception $error){
            $this->entityManager->rollback();
            $response->setError(
                new DTO\Response\Error($error->getMessage(), 1)
            );
        }
    }
    
    public function retornaFatorConversaoProduto($request, $response){
        $row = $request->getRow();
        $produto = $row['CDPRODUTO'];
        $sgunidade = $row['SGUNIDADE'];
        $nrorg = $this->environment->getNrOrgTrab();

        $unidades = $this->tco01900UtilizacaoProdutoReceitaService->retornaFatorConversao($produto, $sgunidade, $nrorg);
        $response->addDataset(new DataSet('UNIDADES', array($unidades)));
    }
    
    public function alteracaoAutomatica($request, $response) {
        try {
            $informaçõesRetornadas = array();
            $this->entityManager->beginTransaction();
            
            $rows = $request->getRow();
            $receitas = $rows['PRODUTOS'];
            $valores = $rows['VALORES'];
          
            $quantidade = $valores['QTPRODPRAT']; //VALOR DIGITADO NO CAMPO
            $alterationForm = $valores["FORMA"];     //SE QUANTIDADE OU PERCENTUAL
            $operationField = $valores["OPERACAO"]; //tipo de OPERAÇÂO se adicionar, incluir ou substituir
          
            $novoProduto = (isset($valores["CDPRODUTO"])) ? $valores["CDPRODUTO"] : "";
            $sgUnidade = (isset($valores["SGUNIDADEPROD"])) ? $valores["SGUNIDADEPROD"] : "";
          
            $vrAprovganho = 100;
            $vrAprovLimpeza = 100;
            $quantidadeLiquida = 0;
            $quantidadeBruta = 0;
            $vrAprovNutr = 100;
            $vrDegelo = 100;
            
            $nrorg = $this->environment->getNrOrgTrab();
            $cdoperador = $this->environment->getCdOperador();
             
            
            foreach($receitas as $receita){
                $cdproduto = $receita['CDPRODUTO'];
                $dadosOriginais = $this->retornaInformacoes($receita, $cdproduto);    
                if($dadosOriginais){
                    $vrAprovganho = 100 + $dadosOriginais["VRPEFATCOC"];
                    if($vrAprovganho <= 0) {
                        $vrAprovganho = 100;
                    }
                    
                    $vrAprovLimpeza = 100 - $dadosOriginais["VRPEFATCOR"];
                    
                    if($vrAprovLimpeza <= 0) {
                        $vrAprovLimpeza = 100;
                    }
                    $quantidadeLiquida = $dadosOriginais["QTLIQUIDA"];
                    $vrAprovNutr =  $dadosOriginais["VRPEFATNUT"];
                    if($vrAprovNutr <= 0) 
                       $vrAprovNutr = 100; 
                    $vrDegelo = 100 - $dadosOriginais["VRPERCDEGELO"];
                    if($vrDegelo <= 0)
                       $vrDegelo = 100; 
                    if(empty($sgUnidade)){
                        $sgUnidade = $dadosOriginais["SGUNIDADE"];
                    }
                } else {
                    if($operationField == "S" || $operationField == "A") {
                        continue;
                    }
                }
                if (!empty($valores['VRPEFATCOCPR']) || $valores['VRPEFATCOCPR'] === 0) {
                    $vrAprovganho = 100 + $valores['VRPEFATCOCPR'];
                }
                if (!empty($valores['VRPEFATCORPR']) || $valores['VRPEFATCORPR'] === 0) {
                    $vrAprovLimpeza = 100 - $valores['VRPEFATCORPR'];
                }
                
                // se for substituica e nao tiver valor, assume qtliquida com valor antigo
                if($operationField == 'I') {
                    $vrAprovganho = 100;
                    $vrAprovLimpeza = 100;
                    $vrDegelo = 100;
                    if (!empty($valores['VRPEFATCOCPR']) || $valores['VRPEFATCOCPR'] === 0) {
                        $vrAprovganho = 100 + $valores['VRPEFATCOCPR'];
                    }
                    if (!empty($valores['VRPEFATCORPR'])|| $valores['VRPEFATCORPR'] === 0) {
                        $vrAprovLimpeza = 100 - $valores['VRPEFATCORPR'];
                    }
                    $quantidadeLiquida = $quantidade;
                    
                } else if($operationField == "S") {
                    if($quantidade > 0) {
                        if($alterationForm == "P") {
                            $quantidadeLiquida = round(($quantidadeLiquida * $quantidade)/100, 5);
                        } else {
                            $quantidadeLiquida = round($quantidade, 5);
                        }
                    }
                    if(empty($sgUnidade)){
                        $sgUnidade = $valores["SGUNIDADE"];
                    }
                } else if($operationField == "A") {
                    $novoProduto = $receita["CDPRODUTO"];
                    if($quantidade > 0) {
                        if($alterationForm == "P") {
                            $quantidadeLiquida = round(($quantidadeLiquida * $quantidade)/100, 5);
                        } else {
                            $quantidadeLiquida = round($quantidade, 5);
                        }
                    }
                }
                
                $quantidadeBruta = ($quantidadeLiquida / $vrAprovganho) * 100;
                $quantidadeBruta = ($quantidadeBruta / $vrDegelo) * 100;
                $quantidadeBruta = round(($quantidadeBruta / $vrAprovLimpeza) * 100, 5);
                
                $fatConv = $this->retornaFatorConversao($novoProduto, $sgUnidade);   

                $qtReq = $quantidadeBruta * $fatConv;
                $qtCalcNutr = $quantidadeBruta - ($qtReq * ((100 - $vrDegelo) / 100));
                $qtCalcNutr = $qtCalcNutr - ($qtCalcNutr * ((100 - $vrAprovLimpeza) / 100));
                $qtCalcNutr = round($qtCalcNutr * $vrAprovNutr/100, 5);
                
                if ($operationField == "S" || $operationField == "I") {
                   $novoProdutoJaNaReceita = $this->retornaInformacoes($receita, $novoProduto);
                   
                   if($novoProdutoJaNaReceita) {
                      if ($operationField == "S")
                         throw new \Exception('PRODUTO_JA_EXISTE_DESTINO');
                      else
                         continue;
                   }
                }
                
                if($quantidadeLiquida < 0.00001 || $quantidadeBruta < 0.00001){
                    throw new \Exception('ERRO_QTD_LIMITE_MINIMO');
                }
                
                $valoresAtualizar = array();
                $valoresAtualizar['TABELA'] = $receita['NMTABLE'];
                $valoresAtualizar['P_CDFILAUXPRAT'] = $receita['CDFILAUXPRAT'];
                $valoresAtualizar['P_CDPRATO'] = $receita['CDPRATO'];
                $valoresAtualizar['P_NOVOPRODUTO'] = $novoProduto;
                $valoresAtualizar['P_CDPRODUTO'] = $cdproduto;
                $valoresAtualizar['P_SGUNIDADE'] = $sgUnidade;
                $valoresAtualizar['P_CDFILIAL'] = $receita['FILIAL'];
                $valoresAtualizar['P_CDSERVFILI'] = $receita['CDSERVFILI'];
                $valoresAtualizar['P_NRPADRAOQUALIDADE'] = $receita['NRPADRAOQUALIDADE'];
                $valoresAtualizar['P_CDREGIAO'] = $receita['CDREGIAO'];
                $valoresAtualizar['P_QTLIQUIDA'] = $quantidadeLiquida;
                $valoresAtualizar['P_QTBRUTA'] = $quantidadeBruta;
                $valoresAtualizar['P_VRPEFATCOC'] = $vrAprovganho - 100;
                $valoresAtualizar['P_VRPEFATCOR'] = 100 - $vrAprovLimpeza;
                $valoresAtualizar['P_QTREQ'] = $qtReq;
                $valoresAtualizar['P_QTCALCNUTR'] = $qtCalcNutr;
                $valoresAtualizar['P_NRORG'] = $nrorg;
                $valoresAtualizar['P_CDOPERADOR'] = $cdoperador;
                
                if($operationField == 'I'){
                  $this->insereReceita($valoresAtualizar);
                } else {
                  $this->atualizaReceita($valoresAtualizar);
                }
            }
            
            $this->entityManager->commit();
           
        }catch(\Exception $error) {
            $this->entityManager->rollback();
            $response->setError(
                new DTO\Response\Error($error->getMessage(), 1)
            );
        }   
    }
    
    private function retornaInformacoes($receitaSelecionada, $cdproduto){
        $tabela = $receitaSelecionada['NMTABLE'];
        $cdfilauxprat = $receitaSelecionada['CDFILAUXPRAT'];
        $cdprato = $receitaSelecionada['CDPRATO'];
        $cdfilial = $receitaSelecionada['FILIAL'];
        $cdservfili = $receitaSelecionada['CDSERVFILI'];
        $nrpadraoqualidade = $receitaSelecionada['NRPADRAOQUALIDADE'];
        $cdregiao = $receitaSelecionada['CDREGIAO'];
        $resultado = false;
        if($tabela == 'RECEPADR'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->retornaRecePadr($cdfilauxprat, $cdprato, $cdproduto);    
        }else if($tabela == 'RECEFILI'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->retornaReceFili($cdfilauxprat, $cdprato, $cdproduto, $cdfilial);
        }else if($tabela == 'RECESERV'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->retornaReceServ($cdfilauxprat, $cdprato, $cdproduto, $cdfilial, $cdservfili);
        }else if($tabela == 'RECEPADQUALI'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->retornaRecePadQuali($cdfilauxprat, $cdprato, $cdproduto, $nrpadraoqualidade);
        }else if($tabela == 'RECEREGIAO'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->retornaReceRegiao($cdfilauxprat, $cdprato, $cdproduto, $nrpadraoqualidade, $cdregiao);
        }else{
            return false;
        }
        return $resultado;
    }
    
    private function retornaFatorConversao($produto, $sgunidade) {
        $fatorConversaoProduto = $this->tco01900UtilizacaoProdutoReceitaService->retornaFatorConversao($produto, $sgunidade, $this->environment->getNrOrgTrab());
        
        if($fatorConversaoProduto) {
            foreach ($fatorConversaoProduto as $fator){
                if($fator['SGUNIDADE'] == $sgunidade){
                    return $fator['VRFATOCONVES'];
                }
            }
            return 1;
        } else {
            return 1;
        }
    }
    
    private function insereReceita($valoresAtualizar){
        $tabela = $valoresAtualizar['TABELA'];
        if($tabela == 'RECEPADR'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->insertRecePadr($valoresAtualizar);    
        }else if($tabela == 'RECEFILI'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->insertReceFili($valoresAtualizar);
        }else if($tabela == 'RECESERV'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->insertReceServ($valoresAtualizar);
        }else if($tabela == 'RECEPADQUALI'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->insertRecePadQuali($valoresAtualizar);
        }else if($tabela == 'RECEREGIAO'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->insertReceRegiao($valoresAtualizar);
        }else{
            return false;
        }    
    }
    
    private function atualizaReceita($valoresAtualizar){
        $tabela = $valoresAtualizar['TABELA'];
        if($tabela == 'RECEPADR'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->updateRecePadr($valoresAtualizar);    
        }else if($tabela == 'RECEFILI'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->updateReceFili($valoresAtualizar);
        }else if($tabela == 'RECESERV'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->updateReceServ($valoresAtualizar);
        }else if($tabela == 'RECEPADQUALI'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->updateRecePadQuali($valoresAtualizar);
        }else if($tabela == 'RECEREGIAO'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->updateReceRegiao($valoresAtualizar);
        }else{
            return false;
        }    
    }
    
    private function deletaReceita($receitaSelecionada){
        $tabela = $receitaSelecionada['NMTABLE'];
        $cdfilauxprat = $receitaSelecionada['CDFILAUXPRAT'];
        $cdprato = $receitaSelecionada['CDPRATO'];
        $cdfilial = $receitaSelecionada['FILIAL'];
        $cdservfili = $receitaSelecionada['CDSERVFILI'];
        $nrpadraoqualidade = $receitaSelecionada['NRPADRAOQUALIDADE'];
        $cdregiao = $receitaSelecionada['CDREGIAO'];
        $cdproduto = $receitaSelecionada['CDPRODUTO'];
        $resultado = false;
        if($tabela == 'RECEPADR'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->deleteRecePadr($cdfilauxprat, $cdprato, $cdproduto);    
        }else if($tabela == 'RECEFILI'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->deleteReceFili($cdfilauxprat, $cdprato, $cdproduto, $cdfilial);
        }else if($tabela == 'RECESERV'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->deleteReceServ($cdfilauxprat, $cdprato, $cdproduto, $cdfilial, $cdservfili);
        }else if($tabela == 'RECEPADQUALI'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->deleteRecePadQuali($cdfilauxprat, $cdprato, $cdproduto, $nrpadraoqualidade);
        }else if($tabela == 'RECEREGIAO'){
            $resultado = $this->tco01900UtilizacaoProdutoReceitaService->deleteReceRegiao($cdfilauxprat, $cdprato, $cdproduto, $nrpadraoqualidade, $cdregiao);
        }else{
            return false;
        }
        return $resultado;    
    }
};